@@ -1,8 +1,5 @@
-# $Id: Reporter.pm 68 2007-06-27 18:11:07Z afoxson $
-# $HeadURL: https://test-reporter.googlecode.com/svn/branches/1.30/lib/Test/Reporter.pm $
-
 # Test::Reporter - sends test results to cpan-testers@perl.org
-# Copyright (c) 2003 Adam J. Foxson. All rights reserved.
+# Copyright (c) 2007 Adam J. Foxson. All rights reserved.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
@@ -20,14 +17,14 @@ use Carp;
 use Net::SMTP;
 use FileHandle;
 use File::Temp;
-use POSIX ();
 use Sys::Hostname;
+use Time::Local ();
 use vars qw($VERSION $AUTOLOAD $Tempfile $Report $DNS $Domain $Send);
 use constant FAKE_NO_NET_DNS => 0;    # for debugging only
 use constant FAKE_NO_NET_DOMAIN => 0; # for debugging only
 use constant FAKE_NO_MAIL_SEND => 0;  # for debugging only
 
-$VERSION = '1.29_04';
+$VERSION = '1.38_01';
 
 local $^W = 1;
 
@@ -45,7 +42,6 @@ sub new {
         '_comments'          => '',
         '_errstr'            => '',
         '_via'               => '',
-        '_mail_send_args'    => '',
         '_timeout'           => 120,
         '_debug'             => 0,
         '_dir'               => '',
@@ -56,6 +52,9 @@ sub new {
             '_osvers'   => $Config{osvers},
             '_myconfig' => Config::myconfig(),
         },
+        '_transport'         => '',
+        '_transport_args'    => [],
+        '_mail_send_args'    => '', # deprecated -> use _transport_args
     };
 
     bless $self, $class;
@@ -109,14 +108,16 @@ sub _process_params {
 
     my %params   = @_;
     my @defaults = qw(
-        mx address grade distribution from comments via timeout debug dir perl_version);
+        mx address grade distribution from comments via timeout debug dir perl_version transport transport_args);
     my %defaults = map {$_ => 1} @defaults;
 
     for my $param (keys %params) {   
         croak __PACKAGE__, ": new: parameter '$param' is invalid." unless
             exists $defaults{$param};
     }
-
+    
+    # XXX need to process transport_args directly rather than through 
+    # the following -- store array ref directly
     for my $param (keys %params) {   
         $self->$param($params{$param});
     }
@@ -146,8 +147,6 @@ sub report {
     $report .= "This distribution has been tested as part of the cpan-testers\n";
     $report .= "effort to test as many new uploads to CPAN as possible.  See\n";
     $report .= "http://testers.cpan.org/\n\n";
-    $report .= "Please cc any replies to cpan-testers\@perl.org to keep other\n";
-    $report .= "test volunteers informed and to prevent any duplicate effort.\n";
 
     if (not $self->{_comments}) {
         $report .= "\n\n--\n\n";
@@ -156,7 +155,7 @@ sub report {
         $report .= "\n--\n" . $self->{_comments} . "\n--\n\n";
     }
 
-    $report .= $self->{_perl_version}->{_myconfig}; # XXX . "\n";
+    $report .= $self->{_perl_version}->{_myconfig};
 
     chomp $report;
     chomp $report;
@@ -183,6 +182,38 @@ sub grade {
     return $self->{_grade} = $grade;
 }
 
+sub transport {
+    my $self = shift;
+    warn __PACKAGE__, ": transport\n" if $self->debug();
+
+    my %transports    = (
+        # support for plugin transports will eventually be added, but not today
+        'Net::SMTP' => 'Builtin transport using Net::SMTP',
+        'Net::SMTP::TLS' => 'Builtin transport using Net::SMTP::TLS',
+        'Mail::Send' => 'Builtin transport using Mail::Send',
+        'HTTP' => 'Builtin transport using HTTP',
+    );
+
+    return $self->{_transport} unless scalar @_;
+
+    my $transport = shift;
+
+    croak __PACKAGE__, ":transport: '$transport' is invalid, choose from: " .
+        join ' ', keys %transports unless $transports{$transport};
+
+    my @args = @_;
+
+    if ( @args && $transport eq 'Mail::Send' && ref $args[0] eq 'ARRAY' ) {
+        # treat as old form of Mail::Send arguments and convert to list
+        $self->transport_args(@{$args[0]});
+    }
+    elsif ( @args ) {
+        $self->transport_args(@args);
+    }
+
+    return $self->{_transport} = $transport;
+}
+
 sub edit_comments {
     my($self, %args) = @_;
     warn __PACKAGE__, ": edit_comments\n" if $self->debug();
@@ -235,13 +266,26 @@ sub send {
         return;
     }
 
-    # Addresses #9831: Usage of Mail::Mailer is broken on Win32
-    if ($^O !~ /^(?:cygwin|MSWin32)$/ && $self->_have_mail_send()) {
+    my $transport = $self->transport();
+
+    if ($transport eq 'Mail::Send' && $self->_have_mail_send()) {
         return $self->_mail_send(@recipients);
     }
-    else {
+    elsif ($transport =~ /^Net::SMTP/ ) {
         return $self->_send_smtp(@recipients);
     }
+    elsif ($transport eq 'HTTP' ) {
+        return $self->_send_http();
+    }
+    else {
+        # Addresses #9831: Usage of Mail::Mailer is broken on Win32
+        if ($^O !~ /^(?:cygwin|MSWin32|VMS)$/ && $self->_have_mail_send()) {
+            return $self->_mail_send(@recipients);
+        }
+        else {
+            return $self->_send_smtp(@recipients);
+        }
+    }
 }
 
 sub write {
@@ -260,7 +304,20 @@ sub write {
     $distribution =~ s/[^A-Za-z0-9\.\-]+//g;
 
     my($fh, $file); unless ($fh = $_[0]) {
-        $file = "$dir/$grade.$distribution.$self->{_perl_version}->{_archname}.$self->{_perl_version}->{_osvers}.${\(time)}.$$.rpt";
+        $file = "$grade.$distribution.$self->{_perl_version}->{_archname}.$self->{_perl_version}->{_osvers}.${\(time)}.$$.rpt";
+
+        if ($^O eq 'VMS') {
+            $file = "$grade.$distribution.$self->{_perl_version}->{_archname}";
+            my $ext = "$self->{_perl_version}->{_osvers}.${\(time)}.$$.rpt";
+            # only 1 period in filename
+            # we also only have 39.39 for filename
+            $file =~ s/\./_/g;
+            $ext  =~ s/\./_/g;
+            $file = $file . '.' . $ext;
+        }
+
+        $file = File::Spec->catfile($dir, $file);
+
         warn $file if $self->debug();
         $fh = FileHandle->new();
         open $fh, ">$file" or die __PACKAGE__, ": Can't open report file '$file': $!";
@@ -346,12 +403,7 @@ sub _mail_send {
     $msg->add('X-Reported-Via', "Test::Reporter ${VERSION}$via");
     $msg->add('Cc', $recipients) if @_;
 
-    if ($self->mail_send_args() and ref $self->mail_send_args() eq 'ARRAY') {
-        $fh = $msg->open(@{$self->mail_send_args()});
-    }
-    else {
-        $fh = $msg->open();
-    }
+    $fh = $msg->open( $self->transport_args() );
 
     print $fh $self->report();
     
@@ -375,16 +427,24 @@ sub _send_smtp {
     my $smtp;
 
     my $mx;
+
+    my $transport = $self->transport;
+
     for my $server (@{$self->{_mx}}) {
-        $smtp = Net::SMTP->new($server, Hello => $helo,
-            Timeout => $self->{_timeout}, Debug => $debug);
+        eval {
+            $smtp = $transport->new($server, Hello => $helo,
+                Timeout => $self->{_timeout}, Debug => $debug,
+                $self->transport_args
+            );
+        };
+        my $err = $@ ? ": $@" : q{};
 
         if (defined $smtp) {
             $mx = $server;
             last;
         }
         else {
-            warn __PACKAGE__, ": Unable to connect to MX '$server'\n" if $self->debug();
+            warn __PACKAGE__, ": Unable to connect to MX '$server'$err\n" if $self->debug();
             $fail++;
         }
     }
@@ -421,37 +481,100 @@ sub _send_smtp {
     my $envelope_sender = $from;
     $envelope_sender =~ s/\s\([^)]+\)$//; # email only; no name
 
-    $success += $smtp->mail($envelope_sender);
-    $success += $smtp->to($self->{_address});
-    $success += $smtp->cc(@recipients) if @recipients;
-    $success += $smtp->data();
-    $success += $smtp->datasend("Date: ", POSIX::strftime("%a, %e %b %Y %T %z", localtime(time)), "\n");
-    $success += $smtp->datasend("Subject: ", $self->subject(), "\n");
-    $success += $smtp->datasend("From: $from\n");
-    $success += $smtp->datasend("To: ", $self->{_address}, "\n");
-    $success += $smtp->datasend("Cc: $recipients\n") if @recipients && $success == 8;
-    $success += $smtp->datasend("Message-ID: ", $self->message_id(), "\n");
-    $success +=
-        $smtp->datasend("X-Reported-Via: Test::Reporter ${VERSION}$via\n");
-    $success += $smtp->datasend("\n");
-    $success += $smtp->datasend($self->report());
-    $success += $smtp->dataend();
-    $success += $smtp->quit;
+    # Net::SMTP::TLS dies on error so eval
+    eval {
+        $success += $smtp->mail($envelope_sender);
+        $success += $smtp->to($self->{_address});
+        $success += $smtp->cc(@recipients) if @recipients;
+        $success += $smtp->data();
+        $success += $smtp->datasend("Date: ", $self->_format_date, "\n");
+        $success += $smtp->datasend("Subject: ", $self->subject(), "\n");
+        $success += $smtp->datasend("From: $from\n");
+        $success += $smtp->datasend("To: ", $self->{_address}, "\n");
+        $success += $smtp->datasend("Cc: $recipients\n") if @recipients && $success == 8;
+        $success += $smtp->datasend("Message-ID: ", $self->message_id(), "\n");
+        $success +=
+            $smtp->datasend("X-Reported-Via: Test::Reporter ${VERSION}$via\n");
+        $success += $smtp->datasend("\n");
+        $success += $smtp->datasend($self->report());
+        $success += $smtp->dataend();
+        $success += $smtp->quit;
+    };
+    my $err = $@;
 
-    if (@recipients) {
-        $self->errstr(__PACKAGE__ .
-            ": Unable to send test report to one or more recipients\n") if $success != 15;
+    if ( $err ) {
+        $self->errstr(__PACKAGE__ . ": Unable to send test report\n");
     }
-    else {
-        $self->errstr(__PACKAGE__ . ": Unable to send test report\n") if $success != 13;
+    elsif (@recipients && $success != 15 ) {
+        $self->errstr(__PACKAGE__ .
+            ": Unable to send test report to one or more recipients\n"); 
+    }
+    elsif ($success != 13) {
+        $self->errstr(__PACKAGE__ . ": Unable to send test report\n");
     }
 
     return $self->errstr() ? 0 : 1;
 }
 
+sub _send_http {
+    my $self = shift;
+    warn __PACKAGE__, ": _send_http\n" if $self->debug();
+    
+    # need LWP
+    eval { require LWP::UserAgent };
+    if ($@) {
+        $self->errstr(__PACKAGE__ . 
+            ": LWP::UserAgent not installed -- can't use HTTP transport\n"
+        );
+        return;
+    }
+
+    # need a transport argument
+    my ($url, $key) = $self->transport_args;
+    if ( ! defined $url ) {
+        $self->errstr(__PACKAGE__ . 
+            ": No url argument provided for HTTP transport\n"
+        );
+        return;
+    }
+
+    # construct the "via"
+    my $via = "Test::Reporter ${VERSION}";
+    $via .= ', via ' . $self->via() if $self->via();
+
+    # post the report
+    my $ua = LWP::UserAgent->new;
+    $ua->timeout(60);
+    $ua->env_proxy;
+
+    my $form = {
+        key => $key,
+        via => $via,
+        from => $self->from(),
+        subject => $self->subject(),
+        report => $self->report(),
+    };
+
+    my $response = $ua->post( $url, $form );
+
+    if ($response->is_success) {
+        return 1;
+    }
+    else {
+        $self->errstr(__PACKAGE__ . 
+            ": HTTP error: ". $response->status_line . "\n" .
+            $response->content
+        );
+    
+        return;
+    }
+}
+
 # Courtesy of Email::MessageID
 sub message_id {
     my $self = shift;
+    warn __PACKAGE__, ": message_id\n" if $self->debug();
+
     my $unique_value = 0;
     my @CHARS = ('A'..'F','a'..'f',0..9);
     my $length = 3;
@@ -494,21 +617,36 @@ sub mx {
     return $self->{_mx};
 }
 
+# Deprecated, but kept for backwards compatibility
+# Passes through to transport_args -- converting from array ref to list to
+# store and converting from list to array ref to get
 sub mail_send_args {
     my $self = shift;
     warn __PACKAGE__, ": mail_send_args\n" if $self->debug();
     croak __PACKAGE__, ": mail_send_args cannot be called unless Mail::Send is installed\n" unless $self->_have_mail_send();
-
     if (@_) {
         my $mail_send_args = shift;
-        croak __PACKAGE__, ": mail_send_args: array reference required" if
-            ref $mail_send_args ne 'ARRAY';
-        $self->{_mail_send_args} = $mail_send_args;
+        croak __PACKAGE__, ": mail_send_args: array reference required\n" 
+            if ref $mail_send_args ne 'ARRAY';
+        $self->transport_args(@$mail_send_args);
+    }
+    return [ $self->transport_args() ];
+}
+
+
+
+sub transport_args {
+    my $self = shift;
+    warn __PACKAGE__, ": transport_args\n" if $self->debug();
+    
+    if (@_) {
+        $self->{_transport_args} = [ @_ ];
     }
 
-    return $self->{_mail_send_args};
+    return @{ $self->{_transport_args} };
 }
 
+
 sub perl_version  {
     my $self = shift;
     warn __PACKAGE__, ": perl_version\n" if $self->debug();
@@ -517,7 +655,12 @@ sub perl_version  {
         my $perl = shift;
         my $q = ( ($^O eq "MSWin32") || ($^O eq 'VMS') ) ? '"' : "'"; # quote for command-line perl
         my $magick = int(rand(1000));                                 # just to check that we get a valid result back
-        my $conf = `$perl -MConfig -e$q print qq{$magick\n\$Config{archname}\n\$Config{osvers}\n},Config::myconfig();$q`;
+        my $cmd  = "$perl -MConfig -e$q print qq{$magick\n\$Config{archname}\n\$Config{osvers}\n},Config::myconfig();$q";
+        if($^O eq 'VMS'){
+            my $sh = $Config{'sh'};
+            $cmd  = "$sh $perl $q-MConfig$q -e$q print qq{$magick\\n\$Config{archname}\\n\$Config{osvers}\\n},Config::myconfig();$q";
+        }
+        my $conf = `$cmd`;
         my %conf;
         ( @conf{ qw( magick _archname _osvers _myconfig) } ) = split( /\n/, $conf, 4);
         croak __PACKAGE__, ": cannot get perl version info from $perl: $conf" if( $conf{magick} ne $magick);
@@ -683,6 +826,10 @@ sub _maildomain {
 
     unless (defined $domain) {
         if ($self->_have_net_domain()) {
+            ###################################################################
+            # The below statement might possibly exhibit intermittent blocking
+            # behavior. Be advised!
+            ###################################################################
             $domain = Net::Domain::domainname();
             undef $domain if $^O eq 'darwin' && $domain =~ /\.local$/;
         }
@@ -730,12 +877,52 @@ sub _realname {
 }
 
 sub _is_a_perl_release {
-    shift; # don't need it..
+    my $self = shift;
+    warn __PACKAGE__, ": _is_a_perl_release\n" if $self->debug();
+
     my $perl = shift;
 
     return $perl =~ /^perl-?\d\.\d/;
 }
 
+
+# Next two subs courtesy of Casey West, Ricardo SIGNES, and Email::Date
+# Visit the Perl Email Project at: http://emailproject.perl.org/
+sub _tz_diff {
+    my $self = shift;
+    warn __PACKAGE__, ": _tz_diff\n" if $self->debug();
+
+    my ($time) = @_;
+
+    my $diff  =   Time::Local::timegm(localtime $time)
+                - Time::Local::timegm(gmtime    $time);
+
+    my $direc = $diff < 0 ? '-' : '+';
+       $diff  = abs $diff;
+    my $tz_hr = int( $diff / 3600 );
+    my $tz_mi = int( $diff / 60 - $tz_hr * 60 );
+
+    return ($direc, $tz_hr, $tz_mi);
+}
+
+sub _format_date {
+    my $self = shift;
+    warn __PACKAGE__, ": _format_date\n" if $self->debug();
+
+    my ($time) = @_;
+    $time = time unless defined $time;
+
+    my ($sec, $min, $hour, $mday, $mon, $year, $wday) = (localtime $time);
+    my $day   = (qw[Sun Mon Tue Wed Thu Fri Sat])[$wday];
+    my $month = (qw[Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec])[$mon];
+    $year += 1900;
+
+    my ($direc, $tz_hr, $tz_mi) = $self->_tz_diff($time);
+
+    sprintf "%s, %d %s %d %02d:%02d:%02d %s%02d%02d",
+      $day, $mday, $month, $year, $hour, $min, $sec, $direc, $tz_hr, $tz_mi;
+}
+
 =head1 NAME
 
 Test::Reporter - sends test results to cpan-testers@perl.org
@@ -773,39 +960,104 @@ Test::Reporter - sends test results to cpan-testers@perl.org
 
 =head1 DESCRIPTION
 
-Test::Reporter reports the test results of any given distribution to the
-CPAN testing service. See B<http://testers.cpan.org/> for details.
+Test::Reporter reports the test results of any given distribution to the CPAN
+Testers. Test::Reporter has wide support for various perl5's and platforms. For
+further information visit the below links:
 
-Test::Reporter has wide support for various perl5's and platforms.
+=over 4
+
+=item * L<http://cpantesters.perl.org/>
+
+CPAN Testers reports (new site)
+
+=item * L<http://testers.cpan.org/>
+
+CPAN Testers reports (old site)
+
+=item * L<http://cpantest.grango.org/>
+
+The new CPAN Testers Wiki (thanks Barbie!)
+
+=item * L<http://lists.cpan.org/showlist.cgi?name=cpan-testers>
+
+The cpan-testers mailing list
+
+=back
+
+Test::Reporter itself--as a project--also has several links for your visiting
+enjoyment:
+
+=over 4
+
+=item * L<http://code.google.com/p/test-reporter/>
+
+Test::Reporter's master project page
+
+=item * L<http://groups.google.com/group/test-reporter>
+
+Discussion group for Test::Reporter
+
+=item * L<http://code.google.com/p/test-reporter/w/list>
+
+The Wiki for Test::Reporter
+
+=item * L<http://repo.or.cz/w/test-reporter.git>
+
+Test::Reporter's public git source code repository.
+
+=item * L<http://search.cpan.org/dist/Test-Reporter/>
+
+Test::Reporter on CPAN
+
+=item * L<http://code.google.com/p/test-reporter/issues/list>
+
+UNFORTUNATELY, WE ARE UNABLE TO ACCEPT TICKETS FILED WITH RT.
+
+Please file all bug reports and enhancement requests at our Google Code issue
+tracker. Thank you for your support and understanding.
+
+=item * L<http://backpan.cpan.org/authors/id/F/FO/FOX/>
+
+=item * L<http://backpan.cpan.org/authors/id/A/AF/AFOXSON/>
+
+If you happen to--for some strange reason--be looking for primordial versions
+of Test::Reporter, you can almost certainly find them at the above 2 links.
+
+=back
 
 =head1 METHODS
 
 =over 4
 
-=item * B<new>
+=item * B<address>
 
-This constructor returns a Test::Reporter object. It will optionally accept
-named parameters for: mx, address, grade, distribution, from, comments,
-via, timeout, debug and dir.
-
-=item * B<subject>
-
-Returns the subject line of a report, i.e.
-"PASS Mail-Freshmeat-1.20 Darwin 6.0". 'grade' and 'distribution' must
-first be specified before calling this method.
-
-=item * B<report>
-
-Returns the actual content of a report, i.e.
-"This distribution has been tested as part of the cpan-testers...". 
-'comments' must first be specified before calling this method, if you have
-comments to make and expect them to be included in the report.
+Optional. Gets or sets the e-mail address that the reports will be
+sent to. By default, this is set to cpan-testers@perl.org. You shouldn't
+need this unless the CPAN Tester's change the e-mail address to send
+report's to.
 
 =item * B<comments>
 
 Optional. Gets or sets the comments on the test report. This is most
 commonly used for distributions that did not pass a 'make test'.
 
+=item * B<debug>
+
+Optional. Gets or sets the value that will turn debugging on or off.
+Debug messages are sent to STDERR. 1 for on, 0 for off. Debugging
+generates very verbose output and is useful mainly for finding bugs
+in Test::Reporter itself.
+
+=item * B<dir>
+
+Optional. Defaults to the current working directory. This method specifies
+the directory that write() writes test report files to.
+
+=item * B<distribution>
+
+Gets or sets the name of the distribution you're working on, for example
+Foo-Bar-0.01. There are no restrictions on what can be put here.
+
 =item * B<edit_comments>
 
 Optional. Allows one to interactively edit the comments within a text
@@ -848,46 +1100,21 @@ result. This must be one of:
   na        distribution will not work on this platform
   unknown   distribution did not include tests
 
-=item * B<distribution>
+=item * B<mail_send_args> -- DEPRECATED
 
-Gets or sets the name of the distribution you're working on, for example
-Foo-Bar-0.01. There are no restrictions on what can be put here.
+Kept for backwards compatibility.  Use C<transport_args> instead.
 
-=item * B<send>
+Optional. If you have MailTools installed and you want to have it
+behave in a non-default manner, parameters that you give this
+method will be passed directly to the constructor of
+Mail::Mailer. See L<Mail::Mailer> and L<Mail::Send> for details.
 
-Sends the test report to cpan-testers@perl.org and cc's the e-mail to the
-specified recipients, if any. If you do specify recipients to be cc'd and
-you do not have Mail::Send installed be sure that you use the author's
-@cpan.org address otherwise they will not be delivered. You must check
-errstr() on a send() in order to be guaranteed delivery. Technically, this
-is optional, as you may use Test::Reporter to only obtain the 'subject' and
-'report' without sending an e-mail at all, although that would be unusual.
+=item * B<message_id>
 
-=item * B<timeout>
-
-Optional. Gets or sets the timeout value for the submission of test
-reports. Default is 120 seconds. 
-
-=item * B<via>
-
-Optional. Gets or sets the value that will be appended to
-X-Reported-Via, generally this is useful for distributions that use
-Test::Reporter to report test results. This would be something
-like "CPANPLUS 0.036".
-
-=item * B<debug>
-
-Optional. Gets or sets the value that will turn debugging on or off.
-Debug messages are sent to STDERR. 1 for on, 0 for off. Debugging
-generates very verbose output and is useful mainly for finding bugs
-in Test::Reporter itself.
-
-=item * B<address>
-
-Optional. Gets or sets the e-mail address that the reports will be
-sent to. By default, this is set to cpan-testers@perl.org. You shouldn't
-need this unless the CPAN Tester's change the e-mail address to send
-report's to.
+Returns an automatically generated Message ID. This Message ID will later
+be included as an outgoing mail header in the test report e-mail. This was
+included to conform to local mail policies at perl.org. This method courtesy
+of Email::MessageID.
 
 =item * B<mx>
 
@@ -900,17 +1127,90 @@ Net::DNS installed it will dynamically retrieve the latest MX's. You
 really shouldn't need to use this unless the hardcoded MX's have
 become wrong and you don't have Net::DNS installed.
 
-=item * B<mail_send_args>
+=item * B<new>
 
-Optional. If you have MailTools installed and you want to have it
-behave in a non-default manner, parameters that you give this
-method will be passed directly to the constructor of
-Mail::Mailer. See L<Mail::Mailer> and L<Mail::Send> for details. 
+This constructor returns a Test::Reporter object. It will optionally accept
+named parameters for: mx, address, grade, distribution, from, comments,
+via, timeout, debug, dir, perl_version, and transport.
 
-=item * B<dir>
+=item * B<perl_version>
 
-Optional. Defaults to the current working directory. This method specifies
-the directory that write() writes test report files to.
+Returns a hashref containing _archname, _osvers, and _myconfig based upon the
+perl that you are using. Alternatively, you may supply a different perl (path
+to the binary) as an argument, in which case the supplied perl will be used as
+the basis of the above data.
+
+=item * B<report>
+
+Returns the actual content of a report, i.e.
+"This distribution has been tested as part of the cpan-testers...". 
+'comments' must first be specified before calling this method, if you have
+comments to make and expect them to be included in the report.
+
+=item * B<send>
+
+Sends the test report to cpan-testers@perl.org and cc's the e-mail to the
+specified recipients, if any. If you do specify recipients to be cc'd and
+you do not have Mail::Send installed be sure that you use the author's
+@cpan.org address otherwise they will not be delivered. You must check
+errstr() on a send() in order to be guaranteed delivery. Technically, this
+is optional, as you may use Test::Reporter to only obtain the 'subject' and
+'report' without sending an e-mail at all, although that would be unusual.
+
+=item * B<subject>
+
+Returns the subject line of a report, i.e.
+"PASS Mail-Freshmeat-1.20 Darwin 6.0". 'grade' and 'distribution' must
+first be specified before calling this method.
+
+=item * B<timeout>
+
+Optional. Gets or sets the timeout value for the submission of test
+reports. Default is 120 seconds. 
+
+=item * B<transport>
+
+Optional. Gets or sets the transport method. If you do not specify a transport,
+one will be selected automatically on your behalf: If you're on Windows,
+Net::SMTP will be selected, if you're not on Windows, Net::SMTP will be
+selected unless Mail::Send is installed, in which case Mail::Send is used.
+
+At the moment, this must be one of either 'Net::SMTP', 'Net::SMTP::TLS',
+'Mail::Send' or 'HTTP'.  You can add additional arguments after the transport
+selection.  These will be passed to the constructor of the lower-level
+transport. This can be used to great effect for all manner of fun and
+enjoyment. ;-) See C<transport_args>.
+
+If L<Net::SMTP::TLS> is used, 'Username' and 'Password' key-value transport
+arguments must be provided.
+
+ $reporter->transport( 
+     'Net::SMTP::TLS', Username => 'jdoe', Password => '123' 
+ );
+
+If the 'HTTP' transport is used, two additional arguments are required: 
+a URL to a L<Test::Reporter::HTTPGateway> compatible server and an (optional)
+API key.
+
+ $reporter->transport( 
+     'HTTP', 'http://example.com/reporter-gateway/', '123456' 
+ );
+
+This is not designed to be an extensible platform upon which to build
+transport plugins. That functionality is planned for the next-generation
+release of Test::Reporter, which will reside in the CPAN::Testers namespace.
+
+=item * B<transport_args>
+
+Optional.  Gets or sets transport arguments that will used in the constructor
+for the selected transport, as appropriate.
+
+=item * B<via>
+
+Optional. Gets or sets the value that will be appended to
+X-Reported-Via, generally this is useful for distributions that use
+Test::Reporter to report test results. This would be something
+like "CPANPLUS 0.036".
 
 =item * B<write and read>
 
@@ -951,33 +1251,14 @@ Mail::Send installed) be sure that you use the author's @cpan.org address
 otherwise they may not be delivered, since the perl.org MX's are unlikely
 to relay for anything other than perl.org and cpan.org.
 
-=head1 BUGS
-
-Unfortunately, we are unable to accept tickets filed with RT. As such, please
-file bug reports and enhancement suggestions at our GoogleCode project site:
-
-  http://code.google.com/p/test-reporter/issues/list
-
-Patches are most welcome. And, where possible, we prefer patches in the
-unified diff format (diff -u). Thank you, kindly.
-
-=head1 EXTERNAL RESOURCES
-
-Project page:
-
-  http://code.google.com/p/test-reporter/
-
-Discussion group:
-
-  http://groups.google.com/group/test-reporter
-
-Subversion repository information page:
-
-  http://code.google.com/p/test-reporter/source
+If you experience a long delay sending mail with Test::Reporter, you may be 
+experiencing a wait as Test::Reporter attempts to determine your email 
+domain.  Setting the MAILDOMAIN environment variable will avoid this delay.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2007 Adam J. Foxson. All rights reserved.
+Copyright (c) 2007 Adam J. Foxson. All rights reserved.  Some revisions
+copyright (c) 2008 David A. Golden.
 
 =head1 LICENSE
 
@@ -994,6 +1275,8 @@ and/or modify it under the same terms as Perl itself.
 
 =item * L<Net::SMTP>
 
+=item * L<Net::SMTP::TLS>
+
 =item * L<File::Spec>
 
 =item * L<File::Temp>
@@ -1014,6 +1297,10 @@ MX's known at the time of this release.
 This is optional. If it's installed Test::Reporter will use Mail::Send
 instead of Net::SMTP.
 
+=item * L<Test::Reporter::HTTPGateway>
+
+This is optional.  It provides a web API for the 'HTTP' transport method.
+
 =back
 
 =head1 AUTHOR
@@ -1024,6 +1311,8 @@ Kirrily "Skud" Robert E<lt>F<skud@cpan.org>E<gt>, and
 Kurt Starsinic E<lt>F<Kurt.Starsinic@isinet.com>E<gt> for predecessor versions
 (CPAN::Test::Reporter, and cpantest respectively).
 
+Additional contributions by David A. Golden E<lt>dagolden@cpan.orgE<gt>.
+
 =cut
 
 1;
